@@ -1,14 +1,18 @@
+from __future__ import annotations
+
 from collections import namedtuple
 from dataclasses import dataclass, field
 from functools import partial
+from typing import Union, Optional, Any, Literal
 
 import numpy as np
 import pandas as pd
 import torch
 from dask.distributed import Client, worker_client
+from scipy.optimize import Bounds, minimize, OptimizeResult
 from symfit import Fit
 from symfit.core.minimizers import DifferentialEvolution, Powell
-from tqdm import trange
+from tqdm.auto import tqdm, trange
 
 from pyhdx.fit_models import (
     SingleKineticModel,
@@ -16,8 +20,9 @@ from pyhdx.fit_models import (
     TwoComponentDissociationModel,
 )
 from pyhdx.fitting_torch import DeltaGFit, TorchFitResult
-from pyhdx.support import temporary_seed
-from pyhdx.models import Protein, HDXMeasurementSet
+from pyhdx.local_cluster import DummyClient
+from pyhdx.support import temporary_seed, pbar_decorator, multiindex_astype
+from pyhdx.models import HDXMeasurementSet, HDXTimepoint, HDXMeasurement
 from pyhdx.config import cfg
 
 EmptyResult = namedtuple("EmptyResult", ["chi_squared", "params"])
@@ -211,6 +216,206 @@ def fit_rates_weighted_average(
     return fit_result
 
 
+def d_uptake_cost_func(x: np.ndarray, A: np.ndarray, b: np.ndarray, d: float) -> float:
+    r"""
+    Cost functions for residue-level D-uptake
+    Calculates ||Ax - b|| + d*regularization
+
+    Where the regularization is the sum of the absolute value of the gradient of x:
+
+    .. math::
+        \frac{d}{N-1}\sum^{N-1}_i |x_i - x_{i+1}|
+
+    Args:
+        x: D-uptake values per residue.
+        A: Coupling matrix ('X'), connecting peptides to residues
+        b: D-uptake values per residue
+        d: regularization parameter
+
+    Returns:
+        Value of the cost function
+
+    """
+    norm = np.mean((A.dot(x) - b) ** 2)
+    reg = d * np.mean(np.abs(np.diff(x)))
+
+    return norm + reg
+
+
+def fit_d_uptake(
+    hdx_obj: Union[HDXMeasurement, HDXTimepoint],
+    guess: Optional[np.ndarray] = None,
+    r1: float = 1.0,
+    bounds: Union[
+        Bounds, list[tuple[Optional[float], Optional[float]]], None, bool
+    ] = True,
+    repeats=10,
+    verbose=True,
+    client: Union[Client, Literal["worker_client"], DummyClient, None] = None,
+) -> DUptakeFitResult:
+    """
+    Fit residue-level D-uptake to a HDX measurement of multiple timepoints or a single HDX
+    timepoint.
+
+    Args:
+        hdx_obj: Input HDX object, either HDXMeasurement or HDXTimepoint.
+        guess: Optional guess array of D-uptake values.
+        r1: Value for r1 regularizer.
+        bounds: Optional bounds. Default is `True`, which are bounds [0, 1] for all elements.
+            Set to `False` or `None` to disable. Custom bounds can be supplied as list of
+            tuples or scipy bounds object.
+        repeats: Number of times to repeat the fit.
+        verbose: Show/hide progress bar
+
+    Returns:
+        D-Uptake fit result object.
+    """
+
+    client = client or DummyClient()
+
+    if isinstance(hdx_obj, HDXMeasurement):
+        Nt = hdx_obj.Nt
+        iterable = hdx_obj
+    elif isinstance(hdx_obj, HDXTimepoint):
+        Nt = 1
+        iterable = [hdx_obj]
+    else:
+        raise TypeError(f"Invalid type for 'hdx_obj': {type(hdx_obj)!r}")
+
+    out = np.empty((Nt, repeats, hdx_obj.Nr))
+    mse_arr = np.empty((Nt, repeats))
+    reg_arr = np.empty((Nt, repeats))
+
+    pbar = tqdm(total=Nt * repeats, disable=not verbose)
+    pbar_wrapper = pbar_decorator(pbar)
+
+    for Ni, hdx_t in enumerate(iterable):
+        X = hdx_t.X
+        d_uptake = hdx_t.data["uptake_corrected"].values
+        pfunc = partial(
+            _fit_single_d_update, X, d_uptake, guess=guess, r1=r1, bounds=bounds
+        )
+        if isinstance(client, DummyClient):
+            pbar_func = pbar_wrapper(pfunc)
+        else:
+            pbar_func = pfunc
+
+        if client == "worker_client":
+            with worker_client() as c:
+                futures = [c.submit(pbar_func, pure=False) for r in range(repeats)]
+                results = c.gather(futures)
+        else:
+            futures = [client.submit(pbar_func, pure=False) for r in range(repeats)]
+            results = client.gather(futures)
+
+        for r, (res, mse_loss, reg_loss) in enumerate(results):
+            out[Ni, r, :] = res.x
+            mse_arr[Ni, r] = mse_loss
+            reg_arr[Ni, r] = reg_loss
+
+            # futures = client.map(pbar_func)
+
+    # if client is None:
+    #     for d, model in zip(d_list, models):
+    #         result = fit_kinetics(hdxm.timepoints, d, model, chisq_thd=chisq_thd)
+    #         results.append(result)
+    # else:
+    #     iterables = [[hdxm.timepoints] * len(d_list), d_list, models]
+    #
+    #     if isinstance(client, Client):
+    #         futures = client.map(fit_kinetics, *iterables, chisq_thd=chisq_thd)
+    #         results = client.gather(futures)
+    #     elif client == "worker_client":
+    #         with worker_client() as client:
+    #             futures = client.map(fit_kinetics, *iterables, chisq_thd=chisq_thd)
+    #             results = client.gather(futures)
+
+    # for Ni, hdx_t in enumerate(iterable):
+    #     for r in range(repeats):
+    #         res, mse_loss, reg_loss = fit_single_d_update(hdx_t, guess=guess, r1=r1, bounds=bounds)
+    #         out[Ni, r, :] = res.x
+    #         mse_arr[Ni, r] = mse_loss
+    #         reg_arr[Ni, r] = reg_loss
+    #
+    #         pbar.update()
+
+    metadata = {"r1": r1, "repeats": repeats}
+    result = DUptakeFitResult(
+        result=out.squeeze(),
+        mse_loss=mse_arr.squeeze(),
+        reg_loss=reg_arr.squeeze(),
+        hdx_obj=hdx_obj,
+        metadata=metadata,
+    )
+
+    return result
+
+    ...
+
+
+def fit_single_d_update(
+    hdx_t: HDXTimepoint,
+    guess: Optional[np.ndarray] = None,
+    r1: float = 1.0,
+    bounds: Union[
+        Bounds, list[tuple[Optional[float], Optional[float]]], None, bool
+    ] = True,
+    **kwargs: Any,
+) -> tuple[OptimizeResult, float, float]:
+    X = hdx_t.X
+    d_uptake = hdx_t.data["uptake_corrected"].values
+
+    return _fit_single_d_update(
+        X, d_uptake, guess=guess, r1=r1, bounds=bounds, **kwargs
+    )
+
+
+def _fit_single_d_update(
+    X: np.ndarray,
+    d_uptake: np.ndarray,
+    guess: Optional[np.ndarray] = None,
+    r1: float = 1.0,
+    bounds: Union[
+        Bounds, list[tuple[Optional[float], Optional[float]]], None, bool
+    ] = True,
+    **kwargs: Any,
+) -> tuple[OptimizeResult, float, float]:
+    """
+    Fit residue-level D-uptake to a single HDX timepoint.
+
+    Args:
+        hdx_t: Input HDXTimepoint object.
+        guess: Optional guess array of D-uptake values.
+        r1: Value for r1 regularizer.
+        bounds: Optional bounds. Default is `True`, which are bounds [0, 1] for all elements.
+            Set to `False` or `None` to disable. Custom bounds can be supplied as list of
+            tuples or scipy bounds object.
+        **kwargs: Additional kwargs to pass to scipy's minimize.
+
+    Returns:
+
+    """
+
+    # X = hdx_t.X
+    # d_uptake = hdx_t.data["uptake_corrected"].values
+    Nr = X.shape[1]  # number of residues / parameters
+
+    if bounds == True:
+        bounds = Bounds(lb=np.zeros(Nr), ub=np.ones(Nr))
+    elif bounds == False:
+        bounds = None
+
+    args = (X, d_uptake, r1)
+    minimize_options = {"method": "L-BFGS-B"}
+    minimize_options.update(kwargs)
+    x0 = guess or np.random.uniform(size=Nr)
+    res = minimize(d_uptake_cost_func, x0, args=args, bounds=bounds, **minimize_options)
+    mse_loss = np.mean((X.dot(res.x) - d_uptake) ** 2)
+    reg_loss = r1 * np.mean(np.abs(np.diff(res.x)))
+
+    return res, mse_loss, reg_loss
+
+
 def fit_rates(hdxm, method="wt_avg", **kwargs):
     """
     Fit observed rates of exchange to HDX-MS data in `hdxm`
@@ -308,7 +513,7 @@ def run_optimizer(
     patience=PATIENCE,
     stop_loss=STOP_LOSS,
     callbacks=None,
-    tqdm=True,
+    verbose=True,
 ):
     """
 
@@ -337,8 +542,8 @@ def run_optimizer(
         Threshold of optimization value below which no progress is made
     callbacks: :obj:`list` or `None`
         List of callback functions
-    tqdm : :obj:`bool`
-        Toggle tqdm progress bar
+    verbose : :obj:`bool`
+        Toggle progress bar
 
     Returns
     -------
@@ -368,7 +573,7 @@ def run_optimizer(
         return loss
 
     stop = 0
-    iter = trange(epochs) if tqdm else range(epochs)
+    iter = trange(epochs) if verbose else range(epochs)
     for epoch in iter:
         optimizer_obj.zero_grad()
         loss = optimizer_obj.step(closure)
@@ -942,3 +1147,112 @@ class RatesFitResult:
         )
 
         return combined_results
+
+
+@dataclass
+class DUptakeFitResult:
+    result: np.ndarray
+    """Array with raw D-uptake fit values, including (guessed/interpolated) D-uptake at 
+        residues without coverage."""
+    mse_loss: np.ndarray
+    reg_loss: np.ndarray
+    hdx_obj: Union[HDXMeasurement, HDXTimepoint]
+    metadata: dict
+
+    @property
+    def d_uptake(self) -> np.ndarray:
+        d = self.result.copy()
+        d[..., ~self.exchanges] = np.nan
+
+        return d
+
+    @property
+    def percentiles(self) -> pd.DataFrame:
+        percentiles = [5, 25, 75, 95]
+        if isinstance(self.hdx_obj, HDXMeasurement):
+            perc = np.percentile(self.result, percentiles, axis=1)
+            perc[..., ~self.exchanges] = np.nan
+            cols = pd.MultiIndex.from_product(
+                [
+                    self.hdx_obj.timepoints,
+                    [f"percentile_{p:02}" for p in percentiles],
+                ],
+                names=["exposure", "quantity"],
+            )
+            reshaped = perc.T.reshape(perc.shape[-1], -1)
+            perc_df = pd.DataFrame(reshaped, columns=cols, index=self.r_number)
+        elif isinstance(self.hdx_obj, HDXTimepoint):
+            perc = np.percentile(self.result, percentiles, axis=0)
+            perc[..., ~self.exchanges] = np.nan
+            perc_df = pd.DataFrame(
+                perc.T,
+                columns=[f"percentile_{p:02}" for p in percentiles],
+                index=self.r_number,
+            )
+
+        return perc_df
+
+    @property
+    def means(self) -> pd.DataFrame:
+        if isinstance(self.hdx_obj, HDXMeasurement):
+            means = self.d_uptake.mean(axis=1)
+            cols = pd.MultiIndex.from_product(
+                [self.hdx_obj.timepoints, ["d_uptake"]], names=["exposure", "quantity"]
+            )
+            means_df = pd.DataFrame(means.T, columns=cols, index=self.r_number)
+
+            return means_df
+
+        elif isinstance(self.hdx_obj, HDXTimepoint):
+            means = self.d_uptake.mean(axis=0)
+            means_df = pd.DataFrame(means, index=self.r_number, columns=["d_uptake"])
+
+            return means_df
+
+    @property
+    def output(self) -> Union[pd.Series, pd.DataFrame]:
+        if isinstance(self.hdx_obj, HDXMeasurement):
+            combined = pd.concat([self.percentiles, self.means], axis=1).sort_index(
+                axis=1, level=0
+            )
+            combined.columns = multiindex_astype(combined.columns, 0, "float")
+            return combined
+
+        elif isinstance(self.hdx_obj, HDXTimepoint):
+            combined = pd.concat([self.percentiles, self.means], axis=1).sort_index(
+                axis=1, level=0
+            )
+
+            return combined
+
+    @property
+    def exchanges(self) -> pd.Series:
+        if isinstance(self.hdx_obj, HDXMeasurement):
+            return self.hdx_obj.coverage["exchanges"]
+        elif isinstance(self.hdx_obj, HDXTimepoint):
+            return self.hdx_obj["exchanges"]
+
+    @property
+    def r_number(self) -> pd.Index:
+        if isinstance(self.hdx_obj, HDXMeasurement):
+            return self.hdx_obj.coverage.r_number
+        elif isinstance(self.hdx_obj, HDXTimepoint):
+            return self.hdx_obj.r_number
+
+
+@dataclass
+class DUptakeFitResultSet(object):
+
+    results: list[DUptakeFitResult]
+
+    @property
+    def output(self) -> pd.DataFrame:
+        names = [result.hdx_obj.name for result in self.results]
+        combined_df = pd.concat(
+            [result.output for result in self.results],
+            axis=1,
+            keys=names,
+            names=["state", "exposure", "quantity"],
+        )
+
+        return combined_df
