@@ -51,7 +51,7 @@ RECT_KWARGS = {"linewidth": 0.5, "linestyle": "-", "edgecolor": "k"}
 
 CBAR_KWARGS = {
     "space": 0,
-    "width": cfg.getfloat("plotting", "cbar_width") / 25.4,
+    "width": cfg.plotting.cbar_width / 25.4,
     "tickminor": True,
 }
 
@@ -75,17 +75,11 @@ def peptide_coverage_figure(
 
     n_subplots = len(subplot_values)
 
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop("nrows", int(np.ceil(n_subplots / ncols)))
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    cbar_width = (
-        figure_kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop(
-        "aspect", cfg.getfloat("plotting", "peptide_coverage_aspect")
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    cbar_width = figure_kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.peptide_coverage_aspect)
 
     cmap = pplt.Colormap(cmap)
     norm = norm or pplt.Norm("linear", vmin=0, vmax=1)
@@ -216,17 +210,11 @@ def residue_time_scatter_figure(
     """per-residue per-exposure values for field  `field` by weighted averaging"""
 
     n_subplots = hdxm.Nt
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop("nrows", int(np.ceil(n_subplots / ncols)))
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop(
-        "aspect", cfg.getfloat("plotting", "residue_scatter_aspect")
-    )
-    cbar_width = (
-        figure_kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.residue_scatter_aspect)
+    cbar_width = figure_kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
 
     cmap = pplt.Colormap(cmap)  # todo allow None as cmap
     norm = norm or pplt.Norm("linear", vmin=0, vmax=1)
@@ -271,7 +259,7 @@ def residue_time_scatter(
     # update cmap, norm defaults
     cmap = pplt.Colormap(cmap)  # todo allow None as cmap
     norm = norm or pplt.Norm("linear", vmin=0, vmax=1)
-    cbar_width = kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
+    cbar_width = kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
 
     scatter_kwargs = {**SCATTER_KWARGS, **kwargs}
     values = hdx_tp.weighted_average(field)
@@ -291,19 +279,13 @@ def residue_scatter_figure(
     **figure_kwargs,
 ):
     n_subplots = hdxm_set.Ns
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop(
         "nrows", int(np.ceil(n_subplots / ncols))
     )  # todo disallow setting rows
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    cbar_width = (
-        figure_kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop(
-        "aspect", cfg.getfloat("plotting", "residue_scatter_aspect")
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    cbar_width = figure_kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.residue_scatter_aspect)
 
     cmap = pplt.Colormap(cmap)
     if norm is None:
@@ -348,7 +330,7 @@ def residue_scatter(
     tps = hdxm.timepoints[np.nonzero(hdxm.timepoints)]
     norm = norm or pplt.Norm("log", tps.min(), tps.max())
 
-    cbar_width = kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
+    cbar_width = kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
     scatter_kwargs = {**SCATTER_KWARGS, **kwargs}
     for hdx_tp in hdxm:
         if isinstance(norm, mpl.colors.LogNorm) and hdx_tp.exposure == 0.0:
@@ -372,12 +354,10 @@ def dG_scatter_figure(
     protein_states = data.columns.get_level_values(0).unique()
 
     n_subplots = len(protein_states)
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop("nrows", int(np.ceil(n_subplots / ncols)))
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop("aspect", cfg.getfloat("plotting", "dG_aspect"))
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.dG_aspect)
     sharey = figure_kwargs.pop("sharey", 1)
 
     cmap_default, norm_default = CMAP_NORM_DEFAULTS["dG"]
@@ -468,12 +448,10 @@ def ddG_scatter_figure(
     combined = pd.concat([ddG, cov], axis=1)
 
     n_subplots = len(protein_states) - 1
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop("nrows", int(np.ceil(n_subplots / ncols)))
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop("aspect", cfg.getfloat("plotting", "dG_aspect"))
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.dG_aspect)
     sharey = figure_kwargs.pop("sharey", 1)
 
     cmap_default, norm_default = CMAP_NORM_DEFAULTS["ddG"]
@@ -535,12 +513,10 @@ def peptide_mse_figure(
     peptide_mse, cmap=None, norm=None, rect_kwargs=None, **figure_kwargs
 ):
     n_subplots = len(peptide_mse.columns.unique(level=0))
-    ncols = figure_kwargs.pop("ncols", min(cfg.getint("plotting", "ncols"), n_subplots))
+    ncols = figure_kwargs.pop("ncols", min(cfg.plotting.ncols, n_subplots))
     nrows = figure_kwargs.pop("nrows", int(np.ceil(n_subplots / ncols)))
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop("aspect", cfg.getfloat("plotting", "peptide_mse_aspect"))
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.peptide_mse_aspect)
 
     cmap = cmap or CMAP_NORM_DEFAULTS["mse"][0]
 
@@ -574,11 +550,9 @@ def peptide_mse_figure(
 def loss_figure(fit_result, **figure_kwargs):
     ncols = 1
     nrows = 1
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
     aspect = figure_kwargs.pop(
-        "aspect", cfg.getfloat("plotting", "loss_aspect")
+        "aspect", cfg.plotting.loss_aspect
     )  # todo loss aspect also in config?
 
     fig, ax = pplt.subplots(
@@ -705,13 +679,9 @@ def linear_bars(
 
     ncols = 1
     nrows = len(hspace) + 1
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop("aspect", cfg.getfloat("plotting", "linear_bars_aspect"))
-    cbar_width = (
-        figure_kwargs.pop("cbar_width", cfg.getfloat("plotting", "cbar_width")) / 25.4
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.linear_bars_aspect)
+    cbar_width = figure_kwargs.pop("cbar_width", cfg.plotting.cbar_width) / 25.4
 
     fig, axes = pplt.subplots(
         nrows=nrows, ncols=ncols, aspect=aspect, width=figure_width, hspace=hspace
@@ -830,12 +800,8 @@ def rainbowclouds_figure(
 
     ncols = 1
     nrows = 1
-    figure_width = (
-        figure_kwargs.pop("width", cfg.getfloat("plotting", "page_width")) / 25.4
-    )
-    aspect = figure_kwargs.pop(
-        "aspect", cfg.getfloat("plotting", "rainbowclouds_aspect")
-    )
+    figure_width = figure_kwargs.pop("width", cfg.plotting.page_width) / 25.4
+    aspect = figure_kwargs.pop("aspect", cfg.plotting.rainbowclouds_aspect)
 
     fig, axes = pplt.subplots(
         nrows=nrows, ncols=ncols, width=figure_width, aspect=aspect, hspace=0
@@ -1012,7 +978,12 @@ def redundancy(ax, hdxm, cmap=None, norm=None):
     img = np.expand_dims(redundancy, 0)
 
     collection = ax.pcolormesh(
-        pplt.edges(x), np.array([0, 1]), img, extend="max", cmap=cmap, norm=norm,
+        pplt.edges(x),
+        np.array([0, 1]),
+        img,
+        extend="max",
+        cmap=cmap,
+        norm=norm,
     )
     ax.format(yticks=[], title="Redundancy")
     return collection
@@ -1030,7 +1001,12 @@ def resolution(ax, hdxm, cmap=None, norm=None):
     img = np.expand_dims(resolution, 0)
 
     collection = ax.pcolormesh(
-        pplt.edges(x), np.array([0, 1]), img, extend="max", cmap=cmap, norm=norm,
+        pplt.edges(x),
+        np.array([0, 1]),
+        img,
+        extend="max",
+        cmap=cmap,
+        norm=norm,
     )
     ax.format(yticks=[], title="Resolution")
     return collection
@@ -1149,6 +1125,8 @@ class ColorTransforms(object):
             "ddG": pplt.Norm("linear", -1e4, 1e4),
             "rfu": pplt.Norm("linear", 0, 1.0, clip=True),
             "drfu": pplt.Norm("linear", -0.5, 0.5, clip=True),
+            "d_uptake": pplt.Norm("linear", 0.0, 1.0, clip=True),
+            "dd_uptake": pplt.Norm("linear", -0.5, 0.5, clip=True),
             "mse": None,
             "foldedness": foldedness_cmap,
         }
@@ -1165,7 +1143,9 @@ class ColorTransforms(object):
             "dG": set_bad(pplt.Colormap(tol_cmap("rainbow_PuRd")).reversed()),
             "ddG": set_bad(tol_cmap("PRGn").reversed()),
             "rfu": set_bad(pplt.Colormap(cc.cm.gouldian)),
-            "drfu": set_bad(pplt.Colormap(cc.cm.diverging_bwr_20_95_c54)),
+            "drfu": set_bad(pplt.Colormap(cc.cm.diverging_bwr_20_95_c54)),  # =CET_D1A
+            "d_uptake": set_bad(pplt.Colormap("Dense")),
+            "dd_uptake": set_bad(pplt.Colormap(cc.cm.diverging_bwr_20_95_c54)),
             "mse": set_bad(pplt.Colormap("cividis"), color="#e3e3e3"),
             "foldedness": foldedness_cmap,
         }
