@@ -1,19 +1,50 @@
 from __future__ import annotations
 
+import contextlib
 import hashlib
-from typing import Iterable, List, Mapping, Any
-
-import numpy as np
 import itertools
 import re
-import contextlib
-from io import StringIO
-from skimage.filters import threshold_multiotsu
-import pandas as pd
-from itertools import count, groupby
 import warnings
+from collections import OrderedDict
+from functools import wraps, reduce
+from io import StringIO
+from itertools import count, groupby
 from pathlib import Path
-from dask.distributed import Client
+from typing import Iterable, Mapping, Any, Literal, Union, TypeVar
+
+import numpy as np
+import numpy.typing as npt
+import pandas as pd
+import typer
+from skimage.filters import threshold_multiotsu
+
+from pyhdx.config import cfg
+
+
+def dataframe_intersection(
+    dataframes: list[pd.DataFrame], by: Union[list, str]
+) -> list[pd.DataFrame]:
+    set_index = [d.set_index(by) for d in dataframes]
+    index_intersection = reduce(pd.Index.intersection, (d.index for d in set_index))
+    intersected = [df.loc[index_intersection].reset_index() for df in set_index]
+
+    return intersected
+
+
+A = TypeVar("A", npt.ArrayLike, pd.Series, pd.DataFrame)
+
+
+def convert_time(
+    values: A, src_unit: Literal["h", "min", "s"], target_unit: Literal["h", "min", "s"]
+) -> A:
+
+    time_lut = {"h": 3600.0, "min": 60.0, "s": 1.0}
+    time_factor = time_lut[src_unit] / time_lut[target_unit]
+
+    if isinstance(values, list):
+        return [v * time_factor for v in values]
+    else:
+        return values * time_factor
 
 
 def make_tuple(item):
@@ -81,12 +112,6 @@ def multiindex_apply_function(
     return new_index
 
 
-def multiindex_astype(index: pd.MultiIndex, level: int, dtype: str) -> pd.MultiIndex:
-
-    new_index = multiindex_apply_function(index, level, "astype", args=[dtype])
-    return new_index
-
-
 def multiindex_set_categories(
     index: pd.MultiIndex,
     level: int,
@@ -105,7 +130,9 @@ def multiindex_set_categories(
 
 
 def multiindex_add_categories(
-    index: pd.MultiIndex, level: int, categories: Any,  # index-like
+    index: pd.MultiIndex,
+    level: int,
+    categories: Any,  # index-like
 ) -> pd.MultiIndex:
     new_index = multiindex_apply_function(
         index, level, "add_categories", args=[categories]
@@ -113,7 +140,11 @@ def multiindex_add_categories(
     return new_index
 
 
-def multiindex_astype(index: pd.MultiIndex, level: int, dtype: str,) -> pd.MultiIndex:
+def multiindex_astype(
+    index: pd.MultiIndex,
+    level: int,
+    dtype: str,
+) -> pd.MultiIndex:
     new_index = multiindex_apply_function(index, level, "astype", args=[dtype])
     return new_index
 
@@ -502,7 +533,7 @@ def rgb_to_hex(rgb_a):
         raise TypeError(f"Invalid type for 'rgb_a': {rgb_a}")
 
     ints = rgba_array.astype(np.uint8).view(dtype=np.uint32).byteswap()
-    padded = np.char.rjust(base_v(ints // 2 ** 8, 16), 6, "0")
+    padded = np.char.rjust(base_v(ints // 2**8, 16), 6, "0")
     result = np.char.add("#", padded).squeeze()
 
     return result
@@ -684,7 +715,7 @@ def multi_otsu(*rates, classes=3):
     all_rates = np.concatenate([data["rate"] for data in rates])
     thd_rates = np.log(all_rates[~np.isnan(all_rates)])
     thds = threshold_multiotsu(thd_rates, classes=classes)
-    return tuple(np.e ** thd for thd in thds)
+    return tuple(np.e**thd for thd in thds)
 
 
 def scale(x, out_range=(-1, 1)):
@@ -726,3 +757,97 @@ def pprint_df_to_file(df, file_path_or_obj):
             file_path_or_obj.write_text(df.__str__())
         elif isinstance(file_path_or_obj, StringIO):
             file_path_or_obj.write(df.__str__())
+
+
+def clean_types(d: Any) -> Any:
+    """cleans up nested dict/list/tuple/other `d` for exporting as yaml
+
+    Converts library specific types to python native types, including numpy dtypes,
+    OrderedDict, numpy arrays
+
+    # https://stackoverflow.com/questions/59605943/python-convert-types-in-deeply-nested-dictionary-or-array
+
+    """
+    if isinstance(d, np.floating):
+        return float(d)
+
+    if isinstance(d, np.integer):
+        return int(d)
+
+    if isinstance(d, np.ndarray):
+        return d.tolist()
+
+    if isinstance(d, list):
+        return [clean_types(item) for item in d]
+
+    if isinstance(d, tuple):
+        return tuple(clean_types(item) for item in d)
+
+    if isinstance(d, OrderedDict):
+        return clean_types(dict(d))
+
+    if isinstance(d, dict):
+        return {k: clean_types(v) for k, v in d.items()}
+
+    else:
+        return d
+
+
+def select_config() -> None:
+    """When the .pyhdx directory has multiple config files, prompts the users
+    for which config to use and subsequently loads it.
+
+    """
+    pyhdx_dir = Path().home() / ".pyhdx"
+    config_options = list(pyhdx_dir.glob("*.yaml"))
+
+    if len(config_options) > 1:
+        s = "Found multiple configuration files:\n"
+        for i, cfg_file in enumerate(config_options, start=1):
+            s += f"{i}: {cfg_file.stem}\n"
+
+        print(s)
+
+        choice = typer.prompt("Which config file to use?", type=int)
+
+        if choice < 1 or choice > len(config_options):
+            print(f"Invalid option: {choice}")
+        else:
+            cfg.load_config(config_options[choice - 1])
+
+
+def pbar_decorator(pbar):
+    """Wraps a progress bar around a function, updating the progress bar with each function call"""
+
+    def func_wrapper(func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            result = func(*args, **kwargs)
+            pbar.update()
+            return result
+
+        return wrapper
+
+    return func_wrapper
+
+
+def array_intersection(
+    arrays: Iterable[np.ndarray], fields: Iterable[str]
+) -> list[np.ndarray]:
+    """
+    Find and return the intersecting entries in multiple arrays.
+
+    Args:
+        arrays: Iterable of input structured arrays
+        fields: Iterable of fields to use to decide if entires are intersecting
+
+    Returns:
+        selected: Output iterable of arrays with only intersecting entries.
+    """
+
+    intersection = reduce(np.intersect1d, [fields_view(d, fields) for d in arrays])
+    selected = [
+        elem[np.isin(fields_view(elem, fields), intersection)] for elem in arrays
+    ]
+
+    return selected
