@@ -105,7 +105,7 @@ class GoldenElvis(object):
 
         return template
 
-    def view(self, view_name, title=None, width=None, height=None, scrollable=True):
+    def view(self, view, title=None, width=None, height=None, scrollable=True):
         """
         Adds a viewable panel.
         :param view: The panel to show in this golden layout sub section.
@@ -119,12 +119,17 @@ class GoldenElvis(object):
         # composing the jinja2 template, we can add them (see compose function).
 
         # It seems that these unique names cannot start with a number or they cannot be referenced directly
-        # Therefore, currently tmpl.main.append cannot be used as this generates
-        fig_panel = self.main_controller.views[view_name]
+        # Therefore, currently tmpl.main.append cannot be used as this generates ?
+        if isinstance(view, str):
+            fig_panel = self.main_controller.views[view]
+            fig_panel.update()  # initialize (needed? probably not)
+
+        else:
+            fig_panel = view
+
         panel_ID = "ID" + str(id(fig_panel))
-        title = default_label_formatter(title or getattr(fig_panel, "name", None))
+        title = title or default_label_formatter(getattr(fig_panel, "name", None))
 
-        fig_panel.update()  # intialize
         item = pn.Row(
             fig_panel.panel, sizing_mode="stretch_both"
         )  # Place figure in layout
