@@ -1,6 +1,6 @@
 """Obtain ΔG for ecSecB tetramer and dimer"""
 from pathlib import Path
-from pyhdx.batch_processing import yaml_to_hdxmset
+from pyhdx.batch_processing import StateParser
 from pyhdx.fileIO import csv_to_dataframe, save_fitresult
 from pyhdx.fitting import fit_gibbs_global_batch
 import yaml
@@ -12,7 +12,8 @@ output_dir = cwd / "test_data" / "output"
 
 yaml_dict = yaml.safe_load(Path(data_dir / "data_states.yaml").read_text())
 
-hdx_set = yaml_to_hdxmset(yaml_dict, data_dir=data_dir)
+parser = StateParser(yaml_dict, data_dir)
+hdx_set = parser.load_hdxmset()
 
 initial_guess_rates = csv_to_dataframe(output_dir / "ecSecB_guess.csv")
 
